# -*- coding: utf-8 -*-

import werkzeug.utils
import re
import requests
import werkzeug

from odoo.addons.auth_signup.controllers.main import AuthSignupHome as Home
from odoo.addons.saas_worker.auth_util import SaasAuth
from odoo.addons.saas_worker.const import SAAS_BASE_URL, SCHEME, TIMEOUT
from odoo.addons.saas_worker.util import from_role, get_country_details, ip2coordinates
from odoo.addons.saas_worker.compat import route, request
from odoo.http import Response


class SignupController(Home):

    @route(website=True, multilang=True)
    def web_auth_signup(self, *args, **kw):
        # hack to make the confirm password condition always true (delete the field from form)
        if request.params and 'password' in request.params:
            request.params['confirm_password'] = request.params['password']

        response = super(SignupController, self).web_auth_signup(*args, **kw)
        scopes = kw.get('scope', '').split()

        def has_scope(*args):
            return any(s in scopes for s in args)

        error = False
        if isinstance(response, Response) and response.is_qweb:
            phone = kw.get('phone')
            if not phone:
                coordinates = ip2coordinates(request.httprequest.environ['REMOTE_ADDR'])
                cc = coordinates['country_code']
                phone = get_country_details(cc).get('phone')
                if phone:
                    phone = '+%s ' % (phone,)
            response.qcontext.update(
                disable_footer=True,
                has_scope=has_scope,
                phone=phone,
            )
            error = 'error' in response.qcontext

        if not error and kw.get('mode') == "signup" and has_scope('trial') and request.httprequest.method == "POST":
            redirect = response.headers.get('location')  # Firefox (see redirect_with_hash)
            if not redirect:
                redirect = re.compile(r'<[^>]+>').sub('', response.data)  # remove html tag
                redirect = redirect.replace('window.location = ', '')  # remove javascript function from redirect_with_hash
            else:
                redirect = "'%s' + location.hash;" % redirect
            return request.redirect(werkzeug.Href("/thanks/signup")(re=redirect))

        return response

    @route(website=True, multilang=True)
    def web_login(self, redirect=None, *args, **kw):
        response = super(SignupController, self).web_login(redirect=redirect, *args, **kw)

        if isinstance(response, Response) and response.is_qweb:
            scopes = kw.get('scope', '').split()

            def has_scope(*args):
                return any(s in scopes for s in args)

            response.qcontext.update(
                disable_footer=True,
                has_scope=has_scope,
            )
        return response

    def _signup_with_values(self, token, values):
        # add geoip coordinates and pricelist in values
        coordinates = ip2coordinates(request.httprequest.environ['REMOTE_ADDR'])
        if isinstance(coordinates, dict):
            # Check if openerp has the country found by geoip
            values['city'] = coordinates['city']
            country_code = coordinates['country_code'].lower()
            country_ids = request.env['res.country'].sudo().search([('code', '=', country_code.upper())]).ids
            if country_ids:
                values['country_id'] = country_ids[0]

        company_size = request.params.get('company_size')
        if company_size:
            Category_sudo = request.env['res.partner.category'].sudo()
            tag_ids = Category_sudo.search([('name', 'ilike', 'Size-%s' % company_size)]).ids
            if not tag_ids:
                tag_ids = [Category_sudo.create({'name': 'Size-%s' % company_size}).id]
            values.setdefault('category_id', []).extend([(4, tag_id) for tag_id in tag_ids])

        if 'phone' in request.params:
            values['phone'] = request.params['phone']
        if 'plan' in request.params:
            values['plan'] = request.params['plan']
        if 'scope' in request.params:
            values['subscription_oauth_scope'] = request.params['scope']

        return super(SignupController, self)._signup_with_values(token, values)

    @from_role('trial')
    @route('/openerp_enterprise/trial-exists', type='json', auth='none')
    def trial_exists(self, login):
        """
        S2S route defined for the sole purpose of accepting an increased request rate from a 'safe'
        client.
        """
        return self._exists(login)

    @route('/openerp_enterprise/exists', type='json', auth='none')
    def exists(self, login):
        return self._exists(login)

    def _exists(self, login):
        """ return whether the login already exists in the database """

        # Speed up search using `=` instead of ilike (login is already lowercased by the form anyway)
        # Note: search with `=` operator does not use unaccent, so indexes should be adapted
        # accordingly, if necessary (i.e. a single direct btree index on the column is enough)
        assert login, "login required"
        login = login.strip().lower()
        res = bool(request.env['res.users'].sudo().search_count([('login', '=', login)]))
        return res

    @route('/openerp_enterprise/confirm_user', type='http', auth='public', website=True)
    def confirm_user(self, db, signup_email, token):
        """This route is targetted by the link of the invitation email to a trial database, when a user with the same
           login exists on odoo.com.
           Accessing this route with the confirmation token will prove that the user has control of the email address.
           A RPC call is made to the database with the login, token and oauth_uid, in order to associate it to the user:
           - the login to identify the user
           - the token to prove that it comes from the email
           - the oauth_uid to set it on the user.
           The call is signed by odoo.com, so the trial server can trust it.
           Afterwards, the user is redirected back to the standard reset password flow.
        """
        if request.website.is_public_user() or request.env.user.login != signup_email:
            login_params = {
                'login': signup_email,
                'redirect': request.httprequest.full_path,
            }
            logout_params = {
                'redirect': f'/web/login?{werkzeug.urls.url_encode(login_params)}'
            }
            redirect_url = f'/web/session/logout?{werkzeug.urls.url_encode(logout_params)}'
            return werkzeug.utils.redirect(redirect_url, 303)

        payload = {"jsonrpc": "2.0", "params": {
            "login": signup_email,
            "token": token,
            "oauth_uid": request.env.user.id,
        }}
        url = f'{SCHEME}://{db}.{SAAS_BASE_URL}/saas_trial/set_oauth_uid'
        resp = requests.post(url, json=payload, auth=SaasAuth(), timeout=TIMEOUT)
        resp.raise_for_status()
        error = resp.json().get("error")
        if error:
            raise Exception("Caught error: %r", error)
        db_uuid = resp.json()['result']['db_uuid']
        return werkzeug.utils.redirect(f'/my/databases/connect/{db_uuid}', 303)

    @from_role('trial', signed=True)
    @route('/openerp_enterprise/reset_passwords', type='json', auth='none')
    def web_auth_reset_passwords(self, passwords_to_reset):
        orig_env_context = request.env.context
        orig_requests_params = request.params
        for kwargs in passwords_to_reset:
            request.update_env(context=orig_env_context)
            if kwargs.get('saas_trial_reset'):
                request.update_context(saas_trial_reset=kwargs['saas_trial_reset'])
            request.params = kwargs
            super(SignupController, self).web_auth_reset_password(**kwargs)
        request.update_env(context=orig_env_context)
        request.params = orig_requests_params

    # csrf set to False to allow a server-to-server reset password
    # So trials can ask a reset password for users using an Odoo.com account to sign in.
    # Inject `saas_trial` in the request env context to know it has been triggered from a trial,
    # to customize the reset password link in order to redirect the user to its saas instance directly.
    # See override of `res.partner._get_signup_url_for_action` in `openerp_enterprise`
    @route(csrf=False)
    def web_auth_reset_password(self, *args, **kw):
        if kw.get('saas_trial_reset'):
            request.env.context = dict(request.env.context, saas_trial_reset=kw['saas_trial_reset'])
        return super(SignupController, self).web_auth_reset_password(*args, **kw)

    def _login_redirect(self, uid, redirect=None):
        if request.params.get('login_success') and not request.params.get('redirect'):
            if (not redirect or redirect == '/my') and uid:  # /my == from website override
                db_user_ids = request.env['openerp.enterprise.oauth.user'].sudo().search([('user_id', '=', uid)])
                if len(db_user_ids.database_id.filtered('alive')):
                    redirect = '/my/databases'
        return super()._login_redirect(uid, redirect=redirect)
